# Copyright (C) 2020-2021
# Author: Cesar Roman
# Contact: cesar@thecesrom.dev

"""Vision Module.

The Vision Module brings robust SCADA visualization to Ignition, giving
you access to real-time and historical data on your HMIs and dashboard
screens.
"""



# Copyright (C) 2020-2021
# Author: Cesar Roman
# Contact: cesar@thecesrom.dev

"""Dataset module."""

__all__ = ["to_json", "to_xml"]

import system.dataset
import system.date
from com.inductiveautomation.ignition.common import BasicDataset
from java.util import Date


def _format_value(obj, header=None):
    """Format the value to be properly represented in JSON.

    Args:
        obj (object): The value to format.
        header (str): Column name used for nested Datasets.

    Returns:
        str: The string representation of the value.
    """
    if obj is None:
        obj = "null"
    elif isinstance(obj, basestring):
        obj = u'"{}"'.format(obj)
    elif isinstance(obj, Date):
        obj = '"{}"'.format(
            system.date.format(obj, "yyyy-MM-dd'T'HH:mm:ss.SSSXXX")
        )
    elif isinstance(obj, BasicDataset):
        obj = _to_json(obj, header, False)
    else:
        obj = "{!r}".format(obj)
    return obj


def _to_json(dataset, root, is_root=True):
    """Return a string JSON representation of the Dataset.

    Private function.

    Args:
        dataset (Dataset): The input dataset.
        root (str): The value of the header.
        is_root (bool): True if we are at the root, False otherwise.
            Optional.

    Returns:
        str: The string JSON representation of the dataset.
    """
    headers = system.dataset.getColumnHeaders(dataset)
    columns = dataset.getColumnCount()
    rows = dataset.getRowCount()
    data = system.dataset.toPyDataSet(dataset)
    ret_str = "{" if is_root else ""
    ret_str += u'"{}":['.format(root)
    col_count = 0

    for row_count, row in enumerate(data, start=1):
        ret_str += "{"
        for header in headers:
            col_count += 1
            val = _format_value(row[header], header)
            comma = "," if col_count < columns else ""
            if isinstance(row[header], BasicDataset):
                ret_str += u"{}{}".format(val, comma)
            else:
                ret_str += u'"{}":{}{}'.format(header, val, comma)
        ret_str += "{}{}".format("}", "," if row_count < rows else "")
        col_count = 0
    ret_str += "]"
    ret_str += "}" if is_root else ""

    return ret_str


def to_json(dataset, root="json"):
    """Return a string JSON representation of the Dataset.

    Args:
        dataset (Dataset): The input dataset.
        root (str): The value of the root. If not provided, it defaults
            to "json". Optional.

    Returns:
        str: The string JSON representation of the dataset.
    """
    return _to_json(dataset, root)


def to_xml(dataset, root="root", element="row"):
    """Return a string XML representation of the Dataset.

    Args:
        dataset (Dataset): The input dataset.
        root (str): The value of the root. If not provided, it defaults
            to "root". Optional.
        element (str): The value of the row. If not provided, it
            defaults to "row". Optional.

    Returns:
        str: The string XML representation of the dataset.
    """
    headers = system.dataset.getColumnHeaders(dataset)
    data = system.dataset.toPyDataSet(dataset)
    new_line = "\n"
    tab = " " * 4
    ret_str = "<{}>{}".format(root, new_line)

    for row in data:
        ret_str += "{}<{}>{}".format(tab, element, new_line)
        for header in headers:
            ret_str += "{0}<{header}>{1}</{header}>{2}".format(
                tab * 2, row[header], new_line, header=header
            )
        ret_str += "{}</{}>{}".format(tab, element, new_line)
    ret_str += "</{}>".format(root)

    return ret_str



# Copyright (C) 2020-2021
# Author: Cesar Roman
# Contact: cesar@thecesrom.dev

"""Utility module."""

__all__ = ["get_function_name", "get_timer", "set_locale", "validate_form"]

import traceback

import system.date
import system.util
from java.util import Date

from incendium import constants


def get_function_name():
    """Get the name of the function last called.

    Returns:
        str: Function's name.
    """
    return traceback.extract_stack(None, 2)[0][2]


def get_timer(date):
    """Get a timer with the time elapsed from value until now.

    This will be in the following format: hh:mm:ss.

    Args:
        date: A date or a date represented in milliseconds.

    Returns:
         str: Time elapsed.
    """
    # Initialize Variables
    date_1 = date if isinstance(date, Date) else system.date.fromMillis(date)
    date_2 = system.date.now()
    minutes, seconds = divmod(system.date.secondsBetween(date_1, date_2), 60)
    hours, minutes = divmod(minutes, 60)
    return "{:02d}:{:02d}:{:02d}".format(hours, minutes, seconds)


def set_locale(user):
    """Set the Locale to the user's default Language.

    If none is configured, the default will be English (US).

    Args:
        user (User): The User.
    """
    locale = constants.DEFAULT_LANGUAGE

    if user is not None and user.get_locale() is not None:
        locale = user.get_locale()

    system.util.setLocale(locale)


def validate_form(strings=None, numbers=None, collections=None):
    """Perform a form validation.

    Args:
        strings (dict): A dictionary containing all strings which must
            not be empty. Optional.
        numbers (dict): A dictionary containing all numbers which must
            be greater than zero. Optional.
        collections (dict): A dictionary containing all collections
            which must at least contain an element. Optional.

    Returns:
        tuple: A tuple containing:
            is_valid (bool): True if all validation tests have passed,
            False otherwise.
            error_message (str): Error message in case any validation
            test has failed.
    """
    # Initialize variables.
    is_valid = True
    error_message = constants.EMPTY_STRING
    counter = 0

    if strings:
        for key, value in strings.iteritems():
            if not value:
                counter += 1
                error_message += (
                    constants.TABBED_LINE + key
                    if counter == 1
                    else constants.NEW_TABBED_LINE + key
                )
                is_valid = False
    if numbers:
        for key, value in numbers.iteritems():
            if value is None or value <= 0:
                counter += 1
                error_message += (
                    constants.TABBED_LINE + key
                    if counter == 1
                    else constants.NEW_TABBED_LINE + key
                )
                is_valid = False
    if collections:
        for key, value in collections.iteritems():
            if value is None or value <= 0:
                counter += 1
                error_message += (
                    constants.TABBED_LINE + key
                    if counter == 1
                    else constants.NEW_TABBED_LINE + key
                )
                is_valid = False

    return is_valid, error_message



# Copyright (C) 2020-2021
# Author: Cesar Roman
# Contact: cesar@thecesrom.dev

"""User module."""

__all__ = [
    "get_emails",
    "get_user",
    "get_user_first_name",
    "get_user_full_name",
]

import system.security
import system.user


class _User(object):
    """Wrapper class for Ignition's User object."""

    def __init__(self, user):
        """User initializer.

        Args:
            user (User): Ignition's user object.
        """
        self._username = user.get(user.Username)
        self._first_name = user.get(user.FirstName)
        self._last_name = user.get(user.LastName)
        self._locale = user.getOrDefault(user.Language)

    def get_first_name(self):
        """Get User's first name.

        Returns:
            str: User's first name.
        """
        return self._first_name

    def get_full_name(self):
        """Get User's full name.

        Returns:
            str: User's full name.
        """
        return " ".join([self._first_name, self._last_name])

    def get_locale(self):
        """Get User's preferred language.

        Returns:
            str: User's preferred language.
        """
        return self._locale


def get_emails(user_source="", filter_role=""):
    """Get a list of email addresses.

    Args:
        user_source (str): The name of the User Source. If not provided,
            the default User Source will be consulted. Optional.
        filter_role (str): The name of the role. If provided, a list of
            email addresses for users that are assigned to a matching
            role will be retrieved, otherwise all email addresses will
            be retrieved. Optional.

    Returns:
        list[str]: A list of email addresses.
    """
    # Initialize variables.
    emails = set()

    # Retrieve users from the User Source.
    users = system.user.getUsers(user_source)

    for user in users:
        _emails = [
            ci.value
            for ci in user.getContactInfo()
            if ci.contactType == "email"
        ]
        for email in _emails:
            if (
                filter_role
                and filter_role in user.getRoles()
                or not filter_role
            ):
                emails.add(email)
    return sorted(list(emails))


def get_user(user_source, failover=None):
    """Look up the logged-in User in a User Source.

    Args:
        user_source (str): The name of the User Source.
        failover (str): The name of the Failover Source. Optional.

    Returns:
        _User: A User object.
    """
    # Initialize variables
    user = None
    user_object = None
    _username = system.security.getUsername()

    # 1 Try SSO
    if user_source:
        user = system.user.getUser(user_source, _username)
    # 2 Try Fallback
    if not user and failover:
        user = system.user.getUser(failover, _username)

    if user:
        user_object = _User(user)

    return user_object


def get_user_first_name(user_source, failover=None):
    """Get the User's First Name.

    Args:
        user_source (str): The name of the Single-Sign-On profile.
        failover (str): The name of the Fallback profile. Optional.

    Returns:
        str: The User's First Name.
    """
    user = get_user(user_source, failover)

    # return value
    return user.get_first_name()


def get_user_full_name(user_source, failover=None):
    """Get the User's Full Name.

    Args:
        user_source (str): The name of the Single-Sign-On profile.
        failover (str): The name of the Fallback profile. Optional.

    Returns:
        str: The User's Full Name.
    """
    user = get_user(user_source, failover)

    # return value
    return user.get_full_name()



# Copyright (C) 2020-2021
# Author: Cesar Roman
# Contact: cesar@thecesrom.dev

"""Constants module."""

# Email settings.
SMTP = "mail.mycompany.com:25"
SENDER = "no-reply@mycompany.com"

# Email templates.
ERROR_REPORT = u"""<html>
    <body>
        <font face='verdana'>
            <b>Error Report.</b><br /><br />
            <b>Error Message</b><br />
            <p>{}</p><br /><br />
            <b>Error Details</b>
            <p>{}</p><br /><br />
            <b>System Details</b><br /><br />
            *** Please do not reply to this email address.
        </font>
    </body>
</html>"""

# Language settings.
DEFAULT_LANGUAGE = "en_US"

# Button's text property.
CANCEL_TEXT = "Cancel"
NO_TEXT = "No"
OK_TEXT = "OK"
YES_TEXT = "Yes"

# Strings.
CANNOT_DELETE_ELEMENT = "This item cannot be deleted."
CANNOT_EDIT_ELEMENT = "This item cannot be edited."
CONFIRM = "Confirm"
EDIT_ERROR = "The following errors occurred:"
EDIT_SUCCESS = "Your changes were saved successfully."
EDIT_SUCCESS_WITH_ERRORS = "Your changes were saved with the following errors:"
EMPTY_STRING = ""
ERROR_WINDOW_TITLE = "Error"
FORM_ERROR = "Please provide information in the following fields:"
INFO_WINDOW_TITLE = "Information"
NEW_LINE = "\n"
NEW_TABBED_LINE = "\n    - "
PROCEED_WITH_ROW_DELETION = (
    "Are you sure you would like to delete the selected row?"
)
PROCEED_WITH_ROWS_DELETION = "Are you sure you would like to delete all rows?"
PROCEED_WITH_SAVING_CHANGES = "Are you sure you would like to proceed?"
PROCEED_WITHOUT_SAVING_CHANGES = (
    "Would you like to proceed without saving your changes?"
)
SUCCESS_WINDOW_TITLE = "Success"
TABBED_LINE = "    - "
UNEXPECTED_ERROR = u"An unexpected error occurred in {}.\n{}"
UNEXPECTED_ERROR_CAUSED_BY = (
    u"An unexpected error occurred in {}.\n{}\nCaused by: {}"
)
WARNING_WINDOW_TITLE = "Warning"



# Copyright (C) 2020-2021
# Author: Cesar Roman
# Contact: cesar@thecesrom.dev

"""GUI module."""

__all__ = ["confirm", "error", "info", "input", "warning"]

import system.util
from javax.swing import JLabel, JOptionPane, JPanel, JTextField

from incendium import constants

# Cursor codes.
CURSOR_DEFAULT = 0
CURSOR_CROSSHAIR = 1
CURSOR_TEXT = 2
CURSOR_WAIT = 3
CURSOR_HAND = 4
CURSOR_MOVE = 5
CURSOR_SW_RESIZE = 6
CURSOR_SE_RESIZE = 7
CURSOR_NW_RESIZE = 8
CURSOR_NE_RESIZE = 9
CURSOR_N_RESIZE = 10
CURSOR_S_RESIZE = 11
CURSOR_W_RESIZE = 12
CURSOR_E_RESIZE = 13


def confirm(message, title="Confirm", show_cancel=False):
    """Display a confirmation dialog box to the user.

    This will present the user with "Yes", "No" and "Cancel" options,
    and a custom message.

    Args:
        message (str): The message to display. This will be translated
            to the selected Locale.
        title (str): A title for the message box. This will be
            translated to the selected Locale. Optional.
        show_cancel (bool): Show a cancel button in the dialog.
            Optional.

    Returns:
        bool: True if the user selected "Yes", False if the user
            selected "No", None if the user selected "Cancel" or
            closes the dialog.
    """
    options = [
        system.util.translate(constants.YES_TEXT),
        system.util.translate(constants.NO_TEXT),
    ]

    if show_cancel:
        options.append(system.util.translate(constants.CANCEL_TEXT))

    choice = JOptionPane.showOptionDialog(
        None,
        system.util.translate(message),
        system.util.translate(title),
        JOptionPane.YES_NO_CANCEL_OPTION,
        JOptionPane.QUESTION_MESSAGE,
        None,
        options,
        options[0],
    )

    return (
        not bool(choice)
        if choice in [JOptionPane.YES_OPTION, JOptionPane.NO_OPTION]
        else None
    )


def error(message, title="Error", detail=None):
    """Display an error-style message box to the user.

    Args:
        message (str): The message to display in an error box. This
            will be translated to the selected Locale.
        title (str): A title for the error box. This will be
            translated to the selected Locale. Optional.
        detail (str): Additional text to display. This will be
            translated to the selected Locale. Optional.
    """
    if detail is None:
        msg = system.util.translate(message)
    else:
        msg = "\n".join(
            [system.util.translate(message), system.util.translate(detail)]
        )
    JOptionPane.showMessageDialog(
        None, msg, system.util.translate(title), JOptionPane.ERROR_MESSAGE
    )


def info(message, title="Information", detail=None):
    """Display an informational-style message popup box to the user.

    Args:
        message (str): The message to display. This will be translated
            to the selected Locale. Will accept html formatting.
        title (str): A title for the message box. This will be
            translated to the selected Locale. Optional.
        detail (str): Additional text to display. This will be
            translated to the selected Locale. Optional.
    """
    if detail is None:
        msg = system.util.translate(message)
    else:
        msg = "\n".join(
            [system.util.translate(message), system.util.translate(detail)]
        )
    JOptionPane.showMessageDialog(
        None,
        msg,
        system.util.translate(title),
        JOptionPane.INFORMATION_MESSAGE,
    )


def input(message, title="Input"):
    """Open up a popup input dialog box.

    This dialog box will show a prompt message, and allow the user to
    type in a string. When the user is done, they can press "OK" or
    "Cancel". If OK is pressed, this function will return with the value
    that they typed in. If Cancel is pressed, this function will return
    the value None.

    Args:
        message (str): The message to display. This will be translated
            to the selected Locale. Will accept html formatting.
        title (str): A title for the input box. This will be translated
            to the selected Locale. Optional.

    Returns:
        str: The string value that was entered in the input box.
    """
    options = [
        system.util.translate(constants.OK_TEXT),
        system.util.translate(constants.CANCEL_TEXT),
    ]

    panel = JPanel()
    label = JLabel("{}: ".format(system.util.translate(message)))
    panel.add(label)
    text_field = JTextField(25)
    panel.add(text_field)

    choice = JOptionPane.showOptionDialog(
        None,
        panel,
        system.util.translate(title),
        JOptionPane.OK_CANCEL_OPTION,
        JOptionPane.PLAIN_MESSAGE,
        None,
        options,
        options[0],
    )

    return text_field.getText() if choice == JOptionPane.OK_OPTION else None


def warning(message, title="Warning", detail=None):
    """Display a message to the user in a warning style popup dialog.

    Args:
        message (str): The message to display in an warning box. This
            will be translated to the selected Locale.
        title (str): A title for the warning box. This will be
            translated to the selected Locale. Optional.
        detail (str): Additional text to display. This will be
            translated to the selected Locale. Optional.
    """
    if detail is None:
        msg = system.util.translate(message)
    else:
        msg = "\n".join(
            [system.util.translate(message), system.util.translate(detail)]
        )
    JOptionPane.showMessageDialog(
        None, msg, system.util.translate(title), JOptionPane.WARNING_MESSAGE
    )



# coding=utf-8
# Copyright (C) 2018-2021
# Author: Cesar Roman
# Contact: cesar@thecesrom.dev

"""Package information."""

__title__ = "incendium"
__description__ = "Package that extends and wraps Ignition Scripting API."
__url__ = "https://github.com/thecesrom/incendium"
__version__ = "1.0.7"
__author__ = "César Román"
__author_email__ = "cesar@thecesrom.dev"
__license__ = "MIT"
__copyright__ = "Copyright (C) 2018-2021 César Román"



# Copyright (C) 2020-2021
# Author: Cesar Roman
# Contact: cesar@thecesrom.dev

"""Date module."""

__all__ = ["compare"]


def compare(date_1, date_2):
    """Compare two dates.

    Args:
        date_1 (Date): The first date.
        date_2 (Date): The second date.

    Returns:
        int: 0 if date_1 and date_2 are equal, -1 If date_2 is greater
            than date_1, 1 If date_1 is greater than date_2.
    """
    ret_val = 1

    if date_1 == date_2:
        ret_val = 0
    elif date_1 < date_2:
        ret_val = -1

    return ret_val



# Copyright (C) 2020-2021
# Authors: Cesar Roman, Kyle Chase.
# Contact: cesar@thecesrom.dev

"""Navigation module."""

__all__ = ["swap_to", "swap_windows"]

import system.nav

# Constants.
CURRENT_DIRECTORY = "."
PARENT_DIRECTORY = ".."
PATH_SEPARATOR = "/"


def _get_full_path(from_path, to_path):
    """Return the full path of the window to swap to.

    The path will be relative to the path of the window to swap from.

    Args:
        from_path: The full path of the window to swap from.
        to_path: The full path or relative path of the window to swap
            to.

    Returns:
        str: The full path of the window to swap to.
    """
    _from = from_path.split(PATH_SEPARATOR)
    _to = to_path.split(PATH_SEPARATOR)
    path_parts = []
    if _to[0] not in (PARENT_DIRECTORY, CURRENT_DIRECTORY):
        path_parts = _to
    elif _to[0] == PARENT_DIRECTORY:
        path_parts = _from[:-2] + _to[1:]
    elif _to[0] == CURRENT_DIRECTORY:
        path_parts = _from[:-1] + _to[1:]

    full_path = []
    for path_part in path_parts:
        if path_part == PARENT_DIRECTORY and full_path:
            full_path = full_path[:-1]
        elif path_part != CURRENT_DIRECTORY:
            full_path.append(path_part)

    return PATH_SEPARATOR.join(full_path)


def swap_to(path, params=None):
    """Perform a window swap.

    This will swap from the current main screen window to the window
    specified.

    Args:
        path (str): The full path or relative path of the window to
            swap to.
        params (dict): A dictionary of parameters to pass into the
            window. The keys in the dictionary must match dynamic
            property names on the target window's root container. The
            values for each key will be used to set those properties.
            Optional.
    """
    swap_windows(system.nav.getCurrentWindow(), path, params)


def swap_windows(from_path, to_path, params=None):
    """Perform a window swap.

    Args:
        from_path (str): The full path of the window to swap from.
        to_path (str): The full path or relative path of the window to
            swap to.
        params (dict): A dictionary of parameters to pass into the
            window. The keys in the dictionary must match dynamic
            property names on the target window's root container. The
            values for each key will be used to set those properties.
            Optional.
    """
    _to_path = _get_full_path(from_path, to_path)
    if _to_path != from_path:
        system.nav.swapWindow(from_path, _to_path, params)



# Copyright (C) 2020-2021
# Author: Cesar Roman
# Contact: cesar@thecesrom.dev

"""Database module."""

__all__ = [
    "DisposableConnection",
    "check",
    "execute_non_query",
    "get_data",
    "get_output_params",
    "get_return_value",
]

import system.db
from java.lang import Thread


class DisposableConnection(object):
    """Disposable Connection.

    A disposable connection enables a database connection in Ignition
    and disables it once the operation is completed to release
    resources.
    """

    def __init__(self, database, retries=3):
        """Disposable Connection initializer.

        Args:
            database (str): The name of the database connection in
                Ignition.
            retries (int): The number of additional times to retry
                enabling the connection. Optional.
        """
        self.database = database
        self.retries = retries

    def __enter__(self):
        """Enter the runtime context related to this object."""
        system.db.setDatasourceEnabled(self.database, True)

        for _ in range(self.retries):
            Thread.sleep(1000)
            if self.status == "Valid":
                break
            if self.status == "Faulted":
                raise IOError(
                    "The database connection {!r} is {}.".format(
                        self.database, self.status
                    )
                )
        else:
            raise IOError(
                "The database connection {!r} could not be enabled.".format(
                    self.database
                )
            )
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Exit the runtime context related to this object."""
        system.db.setDatasourceEnabled(self.database, False)

    @property
    def status(self):
        """Get connection status."""
        connection_info = system.db.getConnectionInfo(self.database)
        return connection_info.getValueAt(0, "Status")


def _execute_sp(
    stored_procedure,
    database="",
    transaction=None,
    skip_audit=False,
    in_params=None,
    out_params=None,
    get_out_params=False,
    get_result_set=False,
    get_ret_val=False,
    return_type_code=None,
    get_update_count=False,
):
    """Execute a stored procedure against the connection.

    Args:
        stored_procedure (str): The name of the stored procedure to
            execute.
        database (str): The name of the database connection to execute
            against. If omitted or "", the project's default database
            connection will be used. Optional.
        transaction (str): A transaction identifier. If omitted, the
            call will be executed in its own transaction. Optional.
        skip_audit (bool): A flag which, if set to True, will cause the
            procedure call to skip the audit system. Useful for some
            queries that have fields which won't fit into the audit log.
            Optional.
        in_params (dict): A Dictionary containing INPUT parameters.
            Optional.
        out_params (dict): A Dictionary containing OUTPUT parameters.
            Optional.
        get_out_params (bool): A flag indicating whether or not to
            return OUTPUT parameters after execution. Optional.
        get_result_set (bool): A flag indicating whether or not to
            return a dataset that is the resulting data of the stored
            procedure, if any. Optional.
        get_ret_val (bool): A flag indicating whether or not to return
            the return value of the stored procedure Call. Optional.
        return_type_code (int): The return value Type Code. Optional.
        get_update_count (bool): A flag indicating whether or not to
            return the number of rows modified by the stored
            procedure, or -1 if not applicable. Optional.

    Returns:
        dict: Result dictionary.
    """
    # Initialize variables.
    _out_params = {}
    result = {
        "output_params": None,
        "result_set": None,
        "return_value": None,
        "update_count": None,
    }

    call = system.db.createSProcCall(
        procedureName=stored_procedure,
        database=database,
        tx=transaction,
        skipAudit=skip_audit,
    )

    # Register INPUT Parameters.
    if in_params is not None:
        for key, value in in_params.iteritems():
            call.registerInParam(key, value[0], value[1])

    # Register OUTPUT Parameters.
    if out_params is not None:
        for key, value in out_params.iteritems():
            call.registerOutParam(key, value)

    # Register RETURN Parameter.
    if get_ret_val:
        call.registerReturnParam(return_type_code)

    # Execute the call.
    system.db.execSProcCall(call)

    if out_params is not None:
        for key in out_params.iterkeys():
            _out_params[key] = call.getOutParamValue(key)

    result["output_params"] = _out_params if get_out_params else None
    result["result_set"] = call.getResultSet() if get_result_set else None
    result["return_value"] = call.getReturnValue() if get_ret_val else None
    result["update_count"] = (
        call.getUpdateCount() if get_update_count else None
    )

    return result


def check(stored_procedure, database="", params=None):
    """Execute a stored procedure against the connection.

    This will return a flag set to TRUE or FALSE.

    Args:
        stored_procedure (str): The name of the stored procedure to
            execute.
        database (str): The name of the database connection to execute
            against. If omitted or "", the project's default database
            connection will be used. Optional.
        params (dict): A Dictionary containing all parameters. Optional.

    Returns:
        bool: The flag.
    """
    output = {"flag": system.db.BIT}
    output_params = get_output_params(
        stored_procedure, output=output, database=database, params=params
    )

    return output_params["flag"]


def execute_non_query(
    stored_procedure, database="", transaction=None, params=None
):
    """Execute a stored procedure against the connection.

    Used for UPDATE, INSERT, and DELETE statements.

    Args:
        stored_procedure (str): The name of the stored procedure to
            execute.
        database (str): The name of the database connection to execute
            against. If omitted or "", the project's default database
            connection will be used. Optional.
        transaction (str): A transaction identifier. If omitted, the
            call will be executed in its own transaction. Optional.
        params (dict): A Dictionary containing all parameters. Optional.

    Returns:
        int: The number of rows modified by the stored procedure, or
            -1 if not applicable.
    """
    result = _execute_sp(
        stored_procedure,
        database=database,
        transaction=transaction,
        in_params=params,
        get_update_count=True,
    )

    return result["update_count"]


def get_data(stored_procedure, database="", params=None):
    """Get data by executing a stored procedure.

    Args:
        stored_procedure (str): The name of the stored procedure to
            execute.
        database (str): The name of the database connection to execute
            against. If omitted or "", the project's default database
            connection will be used. Optional.
        params (dict): A Dictionary containing all parameters. Optional.

    Returns:
        Dataset: A Dataset that is the resulting data of the stored
            procedure call, if any.
    """
    result = _execute_sp(
        stored_procedure,
        database=database,
        in_params=params,
        get_result_set=True,
    )

    return result["result_set"]


def get_output_params(
    stored_procedure, output, database="", transaction=None, params=None
):
    """Get the Output parameters from the Stored Procedure.

    Args:
        stored_procedure (str): The name of the stored procedure to
            execute.
        output (dict): A Dictionary containing all output parameters.
        database (str): The name of the database connection to execute
            against. If omitted or "", the project's default database
            connection will be used. Optional.
        transaction (str): A transaction identifier. If omitted, the
            call will be executed in its own transaction. Optional.
        params (dict): A Dictionary containing all parameters. Optional.

    Returns:
        dict: Result's output_params.
    """
    result = _execute_sp(
        stored_procedure,
        database=database,
        transaction=transaction,
        in_params=params,
        out_params=output,
        get_out_params=True,
    )

    return result["output_params"]


def get_return_value(
    stored_procedure,
    return_type_code,
    database="",
    transaction=None,
    params=None,
):
    """Get the Return Value from the Stored Procedure.

    Args:
        stored_procedure (str): The name of the stored procedure to
            execute.
        return_type_code (int): The Type Code of the Return Value.
        database (str): The name of the database connection to execute
            against. If omitted or "", the project's default database
            connection will be used. Optional.
        transaction (str): A transaction identifier. If omitted, the
            call will be executed in its own transaction. Optional.
        params (dict): A Dictionary containing all parameters. Optional.

    Returns:
        int: The return value.
    """
    result = _execute_sp(
        stored_procedure,
        database=database,
        transaction=transaction,
        in_params=params,
        return_type_code=return_type_code,
        get_ret_val=True,
    )

    return result["return_value"]



# Copyright (C) 2020-2021
# Author: Cesar Roman
# Contact: cesar@thecesrom.dev

"""Tag module."""

__all__ = ["read", "write"]

import system.tag


def read(tag_path):
    """Read the value of the Tags at the given paths.

    Note that this function will block until the read operation is
    complete or times out.

    Args:
        tag_path (str): Reads from the given tag path. If no property is
            specified in the path, the Value property is assumed.

    Returns:
        QualifiedValue: A qualified value. This object has three
            sub-members: value, quality, and timestamp.
    """
    values = system.tag.readBlocking([tag_path])
    return values[0]


def write(tag_path, value):
    """Write a value to a tag.

    Note that this function will block until the write operation is
    complete or times out.

    Args:
        tag_path (str): The path of the tag to write to.
        value (object): The value to write.

    Returns:
        int: 0 if the write failed immediately, 1 if it succeeded
            immediately, and 2 if it is pending.
    """
    quality_codes = system.tag.writeBlocking([tag_path], [value])
    return quality_codes[0].isGood()



# Copyright (C) 2020-2021
# Author: Cesar Roman
# Contact: cesar@thecesrom.dev

"""Net module."""

__all__ = [
    "send_high_priority_email",
    "send_html_email",
    "send_plain_text_email",
]

import system.net

from incendium import constants

HTML_ESCAPE_TABLE = {
    "&": "&amp;",
    '"': "&quot;",
    "'": "&apos;",
    ">": "&gt;",
    "<": "&lt;",
    "\n": "<br />",
    "\t": "&nbsp;" * 4,
}


def _html_escape(text):
    """Escape an HTML file removing traces of offending characters.

    Args:
        text (str): The text to escape.

    Returns:
        str: The escaped text.
    """
    return "".join(HTML_ESCAPE_TABLE.get(c, c) for c in text)


def _send_email(subject, body, html, to, priority):
    """Send an email through the given SMTP server.

    Args:
        subject (str): The subject line for the email.
        body (str): The body text of the email.
        html (bool): A flag indicating whether or not to send the email
            as an HTML email. Will auto-detect if omitted.
        to (list[str]): A list of email addresses to send to.
        priority (str): Priority for the message.
    """
    try:
        system.net.sendEmail(
            smtp=constants.SMTP,
            fromAddr=constants.SENDER,
            subject=subject,
            body=body,
            html=html,
            to=to,
            priority=priority,
        )
    finally:
        pass


def report_error(subject, message, details, to):
    """Send an Error Report email message.

    Args:
        subject (str): The subject line for the email.
        message (str): The error message.
        details (str): The error details.
        to (list[str]): A list of emails addresses to send to.
    """
    body = constants.ERROR_REPORT.format(message, _html_escape(details))
    send_high_priority_email(subject, body, to)


def send_high_priority_email(subject, body, to):
    """Send a High Priority email.

    Args:
        subject (str): The subject line for the email.
        body (str): The body text of the email.
        to (list[str]): A list of email addresses to send to.
    """
    send_html_email(subject, body, to, "1")


def send_html_email(subject, body, to, priority="3"):
    """Send an email in HTML format.

    Args:
        subject (str): The subject line for the email.
        body (str): The body text of the email in HTML format.
        to (list[str]): A list of email addresses to send to.
        priority (str): Priority of the message, from "1" to "5", with
            "1" being highest priority. Defaults to "3" (normal)
            priority. Optional.
    """
    _send_email(subject, body, True, to, priority)


def send_plain_text_email(subject, body, to, priority="3"):
    """Send an email in plain text format.

    Args:
        subject (str): The subject line for the email.
        body (str): The body text of the email in HTML format.
        to (list[str]): A list of email addresses to send to.
        priority (str): Priority of the message, from "1" to "5", with
            "1" being highest priority. Defaults to "3" (normal)
            priority. Optional.
    """
    _send_email(subject, body, False, to, priority)



#!/usr/bin/env python2
# -*- coding: utf-8 -*-

"""incendium."""

import os
from codecs import open

from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(here, "src", "incendium", "__version__.py"), "r") as f:
    exec (f.read(), about)

with open("README.md", "r", "utf-8") as f:
    readme = f.read()

setup(
    name=about["__title__"],
    version=about["__version__"],
    description=about["__description__"],
    long_description=readme,
    long_description_content_type="text/markdown",
    author=about["__author__"],
    author_email=about["__author_email__"],
    url=about["__url__"],
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    license=about["__license__"],
    classifiers=[
        "Intended Audience :: Information Technology",
        "Intended Audience :: Manufacturing",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2 :: Only",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: Implementation :: Jython",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Testing :: Mocking",
    ],
)



# Copyright (C) 2020-2021
# Author: Cesar Roman
# Contact: cesar@thecesrom.dev

"""Exceptions module."""

__all__ = ["ApplicationError", "TagError"]


class ApplicationError(Exception):
    """Application Error class."""

    def __init__(self, message, inner_exception=None, cause=None):
        """Application Error initializer.

        Args:
            message (str): The error message.
            inner_exception (object): The inner Exception. Optional.
            cause (object): The cause of the Exception. Optional.
        """
        self.message = message
        self.inner_exception = inner_exception
        self.cause = cause
        super(ApplicationError, self).__init__(message)

    def __repr__(self):
        """Compute the "official" string representation."""
        return "{}({!r}, {!r}, {!r})".format(
            self.__class__.__name__,
            repr(self.message),
            self.inner_exception.__repr__(),
            repr(self.cause),
        )

    def __str__(self):
        """Compute the "informal" string representation."""
        return repr(self.message)


class TagError(Exception):
    """Tag Error class."""

    def __init__(self, message):
        """Tag Error initializer.

        Args:
            message (str): The error message.
        """
        self.message = message
        super(TagError, self).__init__(message)

    def __repr__(self):
        """Compute the "official" string representation."""
        return "{}({!r})".format(self.__class__.__name__, repr(self.message))

    def __str__(self):
        """Compute the "informal" string representation."""
        return repr(self.message)
